@@ -26,16 +26,6 @@
 
 // Tuning
 
-// This is the size of an individual field in a structure. 1 would
-// lead to e.g. doubles and chars both taking 1 memory address. This
-// is a form of 'compressed' memory, with shrinking and stretching
-// according to the type, when compared to C/C++. On the other hand
-// the normal value of 4 means all fields take 4 memory addresses,
-// as per the norm on a 32-bit machine.
-//
-// Changing this from the default of 4 is deprecated.
-var QUANTUM_SIZE = 4;
-
 // Whether we should add runtime assertions, for example to
 // check that each allocation to the stack does not
 // exceed its size, whether all allocations (stack and static) are
@@ -96,6 +86,8 @@ var TOTAL_MEMORY = 16777216;
 // What malloc()/free() to use, out of
 //  * dlmalloc - a powerful general-purpose malloc
 //  * emmalloc - a simple and compact malloc designed for emscripten
+//  * none     - no malloc() implementation is provided, but you must implement
+//               malloc() and free() yourself.
 // dlmalloc is necessary for multithreading, split memory, and other special
 // modes, and will be used automatically in those cases.
 // In general, if you don't need one of those special modes, and if you don't
@@ -136,6 +128,11 @@ var FAST_UNROLLED_MEMCPY_AND_MEMSET = 1;
 // memory from the system as necessary.
 var ALLOW_MEMORY_GROWTH = 0;
 
+// If ALLOW_MEMORY_GROWTH is true and MEMORY_GROWTH_STEP == -1, memory roughly
+// doubles in size each time is grows. Set MEMORY_GROWTH_STEP to a multiple of
+// WASM page size (64KB), eg. 16MB to enable a slower growth rate.
+var MEMORY_GROWTH_STEP = -1;
+
 // If true, allows more functions to be added to the table at runtime. This is
 // necessary for dynamic linking, and set automatically in that mode.
 var ALLOW_TABLE_GROWTH = 0;
@@ -164,20 +161,6 @@ var GLOBAL_BASE = -1;
 // infinite number.
 var DOUBLE_MODE = 1;
 
-// If enabled, all memory accesses are assumed to be unaligned.  In unaligned
-// memory mode, you can run nonportable code that typically would break in JS
-// (or on ARM for that matter, which also cannot do unaligned reads/writes), at
-// the cost of slowness
-var UNALIGNED_MEMORY = 0;
-
-// If enabled, assumes all reads and writes are fully aligned for the type they
-// use. This is true in proper C code (no undefined behavior), but is sadly
-// common enough that we can't do it by default. See SAFE_HEAP.  For ways to
-// help find places in your code where unaligned reads/writes are done - you
-// might be able to refactor your codebase to prevent them, which leads to
-// smaller and faster code, or even the option to turn this flag on.
-var FORCE_ALIGNED_MEMORY = 0;
-
 // Warn at compile time about instructions that LLVM tells us are not fully
 // aligned.  This is useful to find places in your code where you might refactor
 // to ensure proper alignment.  This is currently only supported in asm.js, not
@@ -222,7 +205,7 @@ var USE_CLOSURE_COMPILER = 0;
 // case, no "var" is created for each export, and instead a loop (of small
 // constant code size, no matter how many exports you have) writes all the
 // exports received into the global scope. Doing so is dangerous since such
-// modifications of the global scope can confuse external JS minifer tools, and
+// modifications of the global scope can confuse external JS minifier tools, and
 // also things can break if the scope the code is in is not the global scope
 // (e.g. if you manually enclose them in a function scope).
 var DECLARE_ASM_MODULE_EXPORTS = 1;
@@ -243,25 +226,6 @@ var SKIP_STACK_IN_SMALL = 1;
 // you do not want inlining
 var INLINING_LIMIT = 0;
 
-// A function size above which we try to automatically break up functions into
-// smaller ones, to avoid the downsides of very large functions (JS engines
-// often compile them very slowly, compile them with lower optimizations, or do
-// not optimize them at all). If 0, we do not perform outlining at all.  To see
-// which funcs are large, you can inspect the source in a debug build (-g2 or -g
-// for example), and can run tools/find_bigfuncs.py on that to get a sorted list
-// by size.  Another possibility is to look in the web console in firefox, which
-// will note slowly-compiling functions.  You will probably want to experiment
-// with various values to see the impact on compilation time, code size and
-// runtime throughput. It is hard to say what values to start testing with, but
-// something around 20,000 to 100,000 might make sense.  (The unit size is
-// number of AST nodes.) Outlining decreases maximum function size, but does so
-// at the cost of increasing overall code size as well as performance (outlining
-// itself makes code less optimized, and requires emscripten to disable some
-// passes that are incompatible with it).
-// Note: For wasm there is usually no need to set OUTLINING_LIMIT, as VMs can
-//       handle large functions well anyhow.
-var OUTLINING_LIMIT = 0;
-
 // Run aggressiveVariableElimination in js-optimizer.js
 var AGGRESSIVE_VARIABLE_ELIMINATION = 0;
 
@@ -305,9 +269,7 @@ var ALIASING_FUNCTION_POINTERS = 0;
 // By default we use a wasm Table for function pointers, which is fast and
 // efficient. When enabling emulation, we also use the Table *outside* the wasm
 // module, exactly as when emulating in asm.js, just replacing the plain JS
-// array with a Table. However, Tables have some limitations currently, like not
-// being able to assign an arbitrary JS method to them, which we have yet to
-// work around.
+// array with a Table.
 var EMULATED_FUNCTION_POINTERS = 0;
 
 // Allows function pointers to be cast, wraps each call of an incorrect type
@@ -596,11 +558,6 @@ var EXPORTED_RUNTIME_METHODS = [];
 // this list lets you add to the default list without modifying it.
 var EXTRA_EXPORTED_RUNTIME_METHODS = [];
 
-// Log all FS operations.  This is especially helpful when you're porting a new
-// project and want to see a list of file system operations happening so that
-// you can create a virtual file system with all of the required files.
-var FS_LOG = 0;
-
 // If set to nonzero, the provided virtual filesystem if treated
 // case-insensitive, like Windows and macOS do. If set to 0, the VFS is
 // case-sensitive, like on Linux.
@@ -661,20 +618,6 @@ var EXPORT_FUNCTION_TABLES = 0;
 
 var RETAIN_COMPILER_SETTINGS = 0;
 
-// this will contain the emscripten version. you should not modify it. This
-// and the following few settings are useful in combination with
-// RETAIN_COMPILER_SETTINGS
-var EMSCRIPTEN_VERSION = '';
-
-// this will contain the optimization level (-Ox). you should not modify it.
-var OPT_LEVEL = 0;
-
-// this will contain the debug level (-gx). you should not modify it.
-var DEBUG_LEVEL = 0;
-
-// Whether we are profiling functions. you should not modify it.
-var PROFILING_FUNCS = 0;
-
 // JS library elements (C functions implemented in JS) that we include by
 // default. If you want to make sure something is included by the JS compiler,
 // add it here.  For example, if you do not use some emscripten_* C API call
@@ -689,7 +632,6 @@ var DEFAULT_LIBRARY_FUNCS_TO_INCLUDE = [
 	'malloc',
 	'free',
 	'emscripten_get_heap_size', // Used by dynamicAlloc() and -s FETCH=1
-	'emscripten_resize_heap' // Used by dynamicAlloc() and -s FETCH=1
 	];
 
 // This list is also used to determine auto-exporting of library dependencies
@@ -748,6 +690,11 @@ var PROXY_TO_WORKER_FILENAME = '';
 // calls pthread_create() to run the application main() in a pthread.  This is
 // something that applications can do manually as well if they wish, this option
 // is provided as convenience.
+//
+// This proxies Module['canvas'], if present, and if OFFSCREEN_CANVAS support
+// is enabled. This has to happen because this is the only chance - this browser
+// main thread does the the only pthread_create call that happens on
+// that thread, so it's the only chance to transfer the canvas from there.
 var PROXY_TO_PTHREAD = 0;
 
 // If set to 1, this file can be linked with others, either as a shared library
@@ -757,7 +704,9 @@ var PROXY_TO_PTHREAD = 0;
 // are linked with.
 //
 // MAIN_MODULE and SIDE_MODULE both imply this, so it not normally necessary
-// to set this explicitly.
+// to set this explicitly. Note that MAIN_MODULE and SIDE_MODULE mode 2 do
+// *not* set this, so that we still do normal DCE on them, and in that case
+// you must keep relevant things alive yourself using exporting.
 var LINKABLE = 0;
 
 // Emscripten 'strict' build mode: Drop supporting any deprecated build options.
@@ -776,19 +725,15 @@ var STRICT = 0;
 var WARN_ON_UNDEFINED_SYMBOLS = 1;
 
 // If set to 1, we will give a link-time error on any undefined symbols (see
-// WARN_ON_UNDEFINED_SYMBOLS). The default value is 1. To allow undefined
-// symbols at link time set this to 0, in which case if an undefined function is
-// called a runtime error will occur.  Any undefined symbols that are listed in
-// EXPORTED_FUNCTIONS will also be reported.
+// WARN_ON_UNDEFINED_SYMBOLS). To allow undefined symbols at link time set this
+// to 0, in which case if an undefined function is called a runtime error will
+// occur.  Any undefined symbols that are listed in EXPORTED_FUNCTIONS will also
+// be reported.
 var ERROR_ON_UNDEFINED_SYMBOLS = 1;
 
-// If set to 1, any -lfoo directives pointing to nonexistent library files will
-// issue a linker error.
-
-// The default value for this is currently 0, but will be transitioned to 1 in
-// the future. To keep relying on building with -s ERROR_ON_MISSING_LIBRARIES=0
-// setting, prefer to set that option explicitly in your build system.
-var ERROR_ON_MISSING_LIBRARIES = 0;
+// My default any -lfoo directives pointing to nonexistent library files will
+// issue a linker error.  Set to 0 to downgrade this to a warning.
+var ERROR_ON_MISSING_LIBRARIES = 1;
 
 // Specifies a list of Emscripten-provided JS libraries to link against.
 // (internal, use -lfoo or -lfoo.js to link to Emscripten system JS libraries)
@@ -836,6 +781,14 @@ var DETERMINISTIC = 0;
 // the module. (This allows, in particular, for you to create multiple
 // instantiations, etc.)
 //
+// The default .html shell file provided in MINIMAL_RUNTIME mode shows
+// an example to how the module is instantiated from within the html file.
+// The default .html shell file provided by traditional runtime mode is only
+// compatible with MODULARIZE=0 mode, so when building with traditional
+// runtime, you should provided your own html shell file to perform the
+// instantiation when building with MODULARIZE=1. (For more details, see
+// https://github.com/emscripten-core/emscripten/issues/7950)
+//
 // If you add --pre-js or --post-js files, they will be included inside
 // the module with the rest of the emitted code. That way, they can be
 // optimized together with it. (If you want something outside of the module,
@@ -917,12 +870,6 @@ var SEPARATE_ASM = 0;
 // with --separate-asm) will contain only the functions you provide.
 var ONLY_MY_CODE = 0;
 
-// Enables profile-guided optimization in the form of runtime checks for which
-// functions are actually called. Emits a list during shutdown that you can pass
-// to DEAD_FUNCTIONS (you can also emit the list manually by calling
-// PGOMonitor.dump());
-var PGO = 0;
-
 // JS library functions on this list are not converted to JS, and calls to them
 // are turned into abort()s. This is potentially useful for reducing code size.
 // If a dead function is actually called, you will get a runtime error.
@@ -931,9 +878,6 @@ var PGO = 0;
 // optimizer pass?
 var DEAD_FUNCTIONS = [];
 
-// If 1, generate an explicit conversion of zext i1 to i32, using ?:
-var EXPLICIT_ZEXT = 0;
-
 // Global variable to export the module as for environments without a
 // standardized module loading system (e.g. the browser and SM shell).
 var EXPORT_NAME = 'Module';
@@ -1029,7 +973,9 @@ var WASM_BACKEND = 0;
 
 // Whether to compile object files as wasm as opposed to the default
 // of using LLVM IR.
-var WASM_OBJECT_FILES = 0;
+// Setting to zero will enable LTO and at link time will also enable bitcode
+// versions of the standard libraries.
+var WASM_OBJECT_FILES = 1;
 
 // An optional comma-separated list of script hooks to run after binaryen,
 // in binaryen's /scripts dir.
@@ -1053,11 +999,24 @@ var BINARYEN_IGNORE_IMPLICIT_TRAPS = 0;
 var BINARYEN_TRAP_MODE = "allow";
 
 // A comma-separated list of passes to run in the binaryen optimizer, for
-// example, "dce,precompute,vacuum".  When set, this overrides the default
-// passes we would normally run.
+// example, "dce,precompute,vacuum".  When set, this overrides/replaces
+// the default passes we would normally run.
+// Note that you can put any binaryen wasm-opt flag here, not just
+// passes. The key thing is that these flags are sent to the main wasm-opt
+// invocation to optimize the wasm binary, which is when we run several
+// important passes. We may also run wasm-opt for various other reasons,
+// like as part of metadce, and these flags are not passed at those times,
+// so they are a bunch of passes (+ other flags) for that one main
+// invocation.
 var BINARYEN_PASSES = "";
 
-// Set the maximum size of memory in the wasm module (in bytes).  Without this,
+// A comma-separated list of passes to run in the binaryen optimizer, like
+// BINARYEN_PASSES, but that is in addition to any default ones. That is,
+// setting this does not override/replace the default passes, and it is
+// appended at the end of the list of passes.
+var BINARYEN_EXTRA_PASSES = "";
+
+// Set the maximum size of memory in the wasm module (in bytes). Without this,
 // TOTAL_MEMORY is used (as it is used for the initial value), or if memory
 // growth is enabled, the default value here (-1) is to have no limit, but you
 // can set this to set a maximum size that growth will stop at.
@@ -1067,12 +1026,10 @@ var WASM_MEM_MAX = -1;
 
 // Whether to compile the wasm asynchronously, which is more efficient and does
 // not block the main thread. This is currently required for all but the
-// smallest modules to run in V8
-var BINARYEN_ASYNC_COMPILATION = 1;
-
-// Directory where we can find Binaryen. Will be automatically set for you, but
-// you can set it to override if you are a Binaryen developer.
-var BINARYEN_ROOT = "";
+// smallest modules to run in chrome.
+//
+// (This option was formerly called BINARYEN_ASYNC_COMPILATION)
+var WASM_ASYNC_COMPILATION = 1;
 
 // WebAssembly defines a "producers section" which compilers and tools can
 // annotate themselves in. Emscripten does not emit this by default, as it
@@ -1116,6 +1073,12 @@ var USE_ICU = 0;
 // 1 = use zlib from emscripten-ports
 var USE_ZLIB = 0;
 
+// 1 = use bzip2 from emscripten-ports
+var USE_BZIP2 = 0;
+
+// 1 = use libjpeg from emscripten-ports
+var USE_LIBJPEG = 0;
+
 // 1 = use libpng from emscripten-ports
 var USE_LIBPNG = 0;
 
@@ -1147,19 +1110,6 @@ var USE_COCOS2D = 0;
 // Formats to support in SDL2_image. Valid values: bmp, gif, lbm, pcx, png, pnm, tga, xcf, xpm, xv
 var SDL2_IMAGE_FORMATS = [];
 
-// Compiler debugging options
-//
-// Some useful items:
-//    framework
-//    frameworkLines
-//    gconst
-//    types
-//    vars
-//    unparsedFunctions
-//    metadata
-//    legalizer
-var DEBUG_TAGS_SHOWING = [];
-
 // The list of defines (C_DEFINES) was moved into struct_info.json in the same
 // directory.  That file is automatically parsed by tools/gen_struct_info.py.
 // If you modify the headers, just clear your cache and emscripten libc should
@@ -1200,8 +1150,6 @@ var PTHREADS_DEBUG = 0;
 
 var MAX_GLOBAL_ALIGN = -1; // received from the backend
 var IMPLEMENTED_FUNCTIONS = []; // received from the backend
-var JSCALL_START_INDEX = 0; // received from the backend
-var JSCALL_SIG_ORDER = {}; // received from the backend
 
 // Duplicate function elimination. This coalesces function bodies that are
 // identical, which can happen e.g. if two methods have different C/C++ or LLVM
@@ -1288,12 +1236,20 @@ var OFFSCREENCANVAS_SUPPORT = 0;
 // back to Offscreen Framebuffer otherwise.
 var OFFSCREEN_FRAMEBUFFER = 0;
 
+// If nonzero, Fetch API (and hence ASMFS) supports backing to IndexedDB. If 0, IndexedDB is not utilized. Set to 0 if
+// IndexedDB support is not interesting for target application, to save a few kBytes.
+var FETCH_SUPPORT_INDEXEDDB = 1;
+
 // If nonzero, prints out debugging information in library_fetch.js
 var FETCH_DEBUG = 0;
 
 // If nonzero, enables emscripten_fetch API.
 var FETCH = 0;
 
+// Internal: name of the file containing the Fetch *.fetch.js, if relevant
+// Do not set yourself.
+var FETCH_WORKER_FILE = '';
+
 // If set to 1, uses the multithreaded filesystem that is implemented within the
 // asm.js module, using emscripten_fetch. Implies -s FETCH=1.
 var ASMFS = 0;
@@ -1315,8 +1271,26 @@ var SINGLE_FILE = 0;
 // to execute the file without the accompanying JS file.
 var EMIT_EMSCRIPTEN_METADATA = 0;
 
-
+//==============================
 // Internal use only, from here
+//==============================
+
+// This will contain the emscripten version. You should not modify this. This
+// and the following few settings are useful in combination with
+// RETAIN_COMPILER_SETTINGS
+var EMSCRIPTEN_VERSION = '';
+
+// This will contain the optimization level (-Ox). You should not modify this.
+var OPT_LEVEL = 0;
+
+// This will contain the debug level (-gx). You should not modify this.
+var DEBUG_LEVEL = 0;
+
+// Whether we are profiling functions. You should not modify this.
+var PROFILING_FUNCS = 0;
+
+// Whether we are emitting a symbol map. You should not modify this.
+var EMIT_SYMBOL_MAP = 0;
 
 // tracks the list of EM_ASM signatures that are proxied between threads.
 var PROXIED_FUNCTION_SIGNATURES = [];
@@ -1363,11 +1337,6 @@ var STATIC_BUMP = -1;
 // the total initial wasm table size.
 var WASM_TABLE_SIZE = 0;
 
-// if set to 1, then generated WASM files will contain a custom
-// "emscripten_metadata" section that contains information necessary
-// to execute the file without the accompanying JS file.
-var EMIT_EMSCRIPTEN_METADATA = 0;
-
 // Tracks whether we are building with errno support enabled. Set to 0
 // to disable compiling errno support in altogether. This saves a little
 // bit of generated code size in applications that do not care about
@@ -1375,6 +1344,12 @@ var EMIT_EMSCRIPTEN_METADATA = 0;
 // for effective code size optimizations to take place.
 var SUPPORT_ERRNO = 1;
 
+// Internal: Tracks whether Emscripten should link in exception throwing (C++ 'throw')
+// support library. This does not need to be set directly, but pass -fno-exceptions
+// to the build disable exceptions support. (This is basically -fno-exceptions, but
+// checked at final link time instead of individual .cpp file compile time)
+var DISABLE_EXCEPTION_THROWING = 0;
+
 // Internal: An array of all symbols exported from asm.js/wasm module.
 var MODULE_EXPORTS = [];
 
@@ -1423,3 +1398,67 @@ var SUPPORT_LONGJMP = 1;
 // there is no "Module.canvas" object, no magic "null" default handling, and DOM element
 // 'target' parameters are taken to refer to CSS selectors, instead of referring to DOM IDs.
 var DISABLE_DEPRECATED_FIND_EVENT_TARGET_BEHAVIOR = 0;
+
+// Specifies whether the generated .html file is run through html-minifier. The set of
+// optimization passes run by html-minifier depends on debug and optimization levels. In
+// -g2 and higher, no minification is performed. In -g1, minification is done, but whitespace
+// is retained. Minification requires at least -O1 or -Os to be used. Pass -s MINIFY_HTML=0
+// to explicitly choose to disable HTML minification altogether.
+var MINIFY_HTML = 1;
+
+// Indicates that the syscalls (which we see statically) indicate that they need full
+// filesystem support. Otherwise, when just a small subset are used, we can get away without
+// including the full filesystem - in particular, if open() is never used, then we don't
+// actually need to support operations on streams.
+var SYSCALLS_REQUIRE_FILESYSTEM = 1;
+
+// A list of feature flags to pass to each binaryen invocation (like wasm-opt, etc.). This
+// is received from wasm-emscripten-finalize, which reads it from the features section.
+var BINARYEN_FEATURES = [];
+
+// Whether EMCC_AUTODEBUG is on, which automatically instruments code for runtime
+// logging that can help in debugging.
+var AUTODEBUG = 0;
+
+// Whether we should use binaryen's wasm2js to convert our wasm to JS. Set when
+// wasm backend is in use with WASM=0 (to enable non-wasm output, we compile to
+// wasm normally, then compile that to JS).
+var WASM2JS = 0;
+
+// Whether we should link in the runtime for ubsan.
+// 0 means do not link ubsan, 1 means link minimal ubsan runtime.
+// This is not meant to be used with `-s`. Instead, to use ubsan, use clang flag
+// -fsanitize=undefined. To use minimal runtime, also pass `-fsanitize-minimal-runtime`.
+var UBSAN_RUNTIME = 0;
+
+// Whether we should load the WASM source map at runtime.
+// This is enabled automatically when using -g4 with sanitizers.
+var LOAD_SOURCE_MAP = 0;
+
+// Whether embind has been enabled.
+var EMBIND = 0;
+
+// Legacy settings that have been removed or renamed.
+// For renamed settings the format is:
+// [OLD_NAME, NEW_NAME]
+// For removed settings (which now effectively have a fixed value and can no
+// longer be changed) the format is:
+// [OPTION_NAME, POSSIBLE_VALUES, ERROR_EXPLANATION], where POSSIBLE_VALUES is
+// an array of values that will still be silently accepted by the compiler.
+// First element in the list is the canonical/fixed value going forward.
+// This allows existing build systems to keep specifying one of the supported
+// settings, for backwards compatibility.
+var LEGACY_SETTINGS = [
+  ['BINARYEN_ASYNC_COMPILATION', 'WASM_ASYNC_COMPILATION'],
+  ['UNALIGNED_MEMORY', [0], 'forced unaligned memory not supported in fastcomp'],
+  ['FORCE_ALIGNED_MEMORY', [0], 'forced aligned memory is not supported in fastcomp'],
+  ['PGO', [0], 'pgo no longer supported'],
+  ['QUANTUM_SIZE', [4], 'altering the QUANTUM_SIZE is not supported'],
+  ['FUNCTION_POINTER_ALIGNMENT', [2], 'Starting from Emscripten 1.37.29, no longer available (https://github.com/emscripten-core/emscripten/pull/6091)'],
+  ['BUILD_AS_SHARED_LIB', [0], 'Starting from Emscripten 1.38.16, no longer available (https://github.com/emscripten-core/emscripten/pull/7433)'],
+  ['SAFE_SPLIT_MEMORY', [0], 'Starting from Emscripten 1.38.19, SAFE_SPLIT_MEMORY codegen is no longer available (https://github.com/emscripten-core/emscripten/pull/7465)'],
+  ['SPLIT_MEMORY', [0], 'Starting from Emscripten 1.38.19, SPLIT_MEMORY codegen is no longer available (https://github.com/emscripten-core/emscripten/pull/7465)'],
+  ['BINARYEN_METHOD', ['native-wasm'], 'Starting from Emscripten 1.38.23, Emscripten now always builds either to Wasm (-s WASM=1 - default), or to asm.js (-s WASM=0), other methods are not supported (https://github.com/emscripten-core/emscripten/pull/7836)'],
+  ['PRECISE_I64_MATH', [1, 2], 'Starting from Emscripten 1.38.26, PRECISE_I64_MATH is always enabled (https://github.com/emscripten-core/emscripten/pull/7935)'],
+  ['MEMFS_APPEND_TO_TYPED_ARRAYS', [1], 'Starting from Emscripten 1.38.26, MEMFS_APPEND_TO_TYPED_ARRAYS=0 is no longer supported. MEMFS no longer supports using JS arrays for file data (https://github.com/emscripten-core/emscripten/pull/7918)'],
+];
